@@ -22,17 +22,12 @@ import pickle
 import cPickle
 import pickletools
 import thread
-import threading
 import os.path
 import collections
 import operator
 import functools
 import inspect
 import copy_reg
-
-#saved_dispatch = pickle.Pickler.dispatch.copy()
-#saved_dispatch_table = pickle.dispatch_table.copy()
-
 import types
 from pickle import PickleError, PicklingError
 try:
@@ -47,6 +42,7 @@ import tempfile
 import codecs
 import weakref
 
+
 try:
     from stackless._wrap import function as STACKLESS_FUNCTION_WRAPPER
 except ImportError:
@@ -96,60 +92,15 @@ try:
 except ImportError:
     class CSTRINGIO_INPUT_TYPE(object): pass
     class CSTRINGIO_OUTPUT_TYPE(object): pass
-    
-class _DelayedLogger(object):
-    from imp import lock_held
-    
-    def __init__(self, name):
-        self.__logger = None
-        self.__name = name
-        self.__tl = threading.local()
-    
-    def __getLogger(self):
-        if self.__logger is None:
-            import logging
-            self.__logger = logging.getLogger(self.__name)
-        return self.__logger
-        
-    def __getQueue(self):
-        tl = self.__tl
-        try:
-            return tl.queue
-        except AttributeError:
-            tl.queue = collections.deque()
-        return tl.queue
-        
-    def __call__(self):
-        if self.lock_held():
-            return self
-        self.flush()
-        return self.__getLogger()
-    
-    def __getattr__(self, name):
-        f = functools.partial(self.__log, name)
-        setattr(self, name, f)
-        return f
-    
-    def __log(self, method, *args, **kw):
-        if method is not None:
-            queue = self.__getQueue()
-            queue.append((method, args, kw))
-        
-        if not self.lock_held():
-            queue = self.__getQueue()
-            while True:
-                try:
-                    nextCall = queue.popleft()
-                except IndexError:
-                    break
-                getattr(self.__getLogger(), nextCall[0])(*nextCall[1], **nextCall[2])
-                
-    def flush(self):
-        self.__log(None)
 
-LOGGER = _DelayedLogger(__name__)
-            
-            
+__LOGGER = None
+def LOGGER():
+    global __LOGGER
+    if __LOGGER is None:
+        import logging
+        __LOGGER = logging.getLogger(__name__)
+    return __LOGGER
+
 
 # types to be replaced by proxies, if possible  , 
 RESOURCE_TYPES = (file, socket.SocketType, SOCKET_PAIR_TYPE, socket._closedsocket,
@@ -220,13 +171,6 @@ def restore_modules_entry(doDel, old, new, preserveReferenceToNew=True):
         release_lock()
     return new
 
-#def import_module(name):
-#    global sys
-#    if sys is None:
-#        sys = __import__('sys')
-#    __import__(name)
-#    return sys.modules[name]
-
 def create_thread_lock(locked):
     """recreate a lock object"""
     l = thread.allocate_lock()
@@ -391,6 +335,7 @@ class Pickler(pickle.Pickler):
         if serializeableModules is None:
             serializeableModules = []
         self.serializeableModules = serializeableModules
+        self._serializableModulesIds = set()
         
         if mangleModuleName is not None:
             if not callable(mangleModuleName):
@@ -462,9 +407,18 @@ class Pickler(pickle.Pickler):
         
     def mustSerialize(self, obj):
         """test, if a module must be serialised"""
+
+        objId = id(obj)
+        if objId in self._serializableModulesIds:
+            return True
         # Legacy check for flowGuide2 workflow modules
         try:
-            return bool(getattr(obj, MODULE_TO_BE_PICKLED_FLAG_NAME))
+            ret = bool(getattr(obj, MODULE_TO_BE_PICKLED_FLAG_NAME))
+            if ret:
+                self._serializableModulesIds.add(objId)
+            # it is intended, that an module can be marked as importable by setting
+            # MODULE_TO_BE_PICKLED_FLAG_NAME to False
+            return ret
         except Exception:
             pass # Attribute is not present
         
@@ -482,12 +436,7 @@ class Pickler(pickle.Pickler):
                         break
         else:
             return False
-        try:
-            # Mark the module. The mark will be pickled and therefore 
-            # the unpickled module will be considered beeing "mustSerialize" too 
-            setattr(obj, MODULE_TO_BE_PICKLED_FLAG_NAME, True)
-        except Exception, e:
-            LOGGER().debug("Ohh, module %r is not writable: %r", obj.__name__, e)
+        self._serializableModulesIds.add(objId)
         return True
         
 
@@ -563,14 +512,15 @@ class Pickler(pickle.Pickler):
             return
 
         try:
-            objDict = getattr(obj, "__dict__", None)
+            objDict = obj.__dict__
         except Exception:
-            objDict = None
-        if isinstance(objDict, types.DictType):
-            dictId = id(objDict)
-            x = self.memo.get(dictId)
-            if x is not None:
-                raise ObjectAlreadyPickledError("__dict__ already pickled (memo %s) for %r" % (x[0], obj), obj, dictId)
+            pass
+        else:
+            if isinstance(objDict, types.DictType):
+                dictId = id(objDict)
+                x = self.memo.get(dictId)
+                if x is not None:
+                    raise ObjectAlreadyPickledError("__dict__ already pickled (memo %s) for %r" % (x[0], obj), obj, dictId)
 
 
         # special cases 
@@ -784,13 +734,13 @@ class Pickler(pickle.Pickler):
     def save_global(self, obj, name=None, pack=struct.pack):
         write = self.write
         memo = self.memo
-        t = type(obj)
+        isClass = inspect.isclass(obj)
 
         if name is None:
             name = obj.__name__
 
         module = None
-        if t is type:
+        if isClass:
             for k in dir(types):
                 if obj is getattr(types, k, None):
                     module = types.__name__
@@ -803,11 +753,16 @@ class Pickler(pickle.Pickler):
                 module = pickle.whichmodule(obj, name)
 
         try:
-            __import__(module)
             mod = sys.modules[module]
+            if isinstance(mod, types.ModuleType) and self.mustSerialize(mod) and memo.get(id(mod)) is not None:
+                # pickling of the module is in progress.
+                # We must not import from this module
+                if isClass:
+                    return self.saveClass(obj)
+                raise pickle.PicklingError("Can't pickle %r: the containing module '%s' must be serialized by value" % (obj, module))
             klass = getattr(mod, name)
-        except (ImportError, KeyError, AttributeError):
-            if t in (type, types.ClassType):
+        except (KeyError, AttributeError):
+            if isClass:
                 return self.saveClass(obj)
             raise pickle.PicklingError(
                 "Can't pickle %r: it's not found as %s.%s" %
@@ -819,7 +774,7 @@ class Pickler(pickle.Pickler):
                     # Sys contains two copies of these functions, one 
                     # named <name> and the other named "__<name>__".  
                     return self.save_global(obj, "__"+name+"__", pack)
-                if t in (type, types.ClassType):
+                if isClass:
                     return self.saveClass(obj)
                 raise pickle.PicklingError(
                     "Can't pickle %r: it's not the same object as %s.%s" %
@@ -1146,8 +1101,9 @@ class Pickler(pickle.Pickler):
     def saveModule(self, obj, reload=False):
         write = self.write
 
-        self.module_dict_ids[id(obj.__dict__)] = obj
         obj_name = obj.__name__
+        obj_dict = obj.__dict__
+        self.module_dict_ids[id(obj_dict)] = obj
         if obj is not sys.modules.get(obj_name):
             # either an anonymous module or it did change its __name__
             for k,v in sys.modules.iteritems():
@@ -1194,20 +1150,20 @@ class Pickler(pickle.Pickler):
             self.save(restore_modules_entry)
             self.write(pickle.TRUE if doDel else pickle.FALSE)
             self.save_reduce(save_modules_entry, (pickledModuleName,))
-            
-        savedModule = save_modules_entry(obj_name)
-        try:            
-            # create the module
-            self.save_reduce(create_module, (type(obj), pickledModuleName, getattr(obj, "__doc__", None)), obj.__dict__, obj=obj)
-        finally:
-            restore_modules_entry(True, savedModule, obj, preserveReferenceToNew=False)
-            # Flush log messages
-            LOGGER.flush()
-            
+
+        module_dict = dict(obj_dict)
+        try:
+            del module_dict['__doc__'] # saved separately
+        except KeyError:
+            pass
+        module_dict[MODULE_TO_BE_PICKLED_FLAG_NAME] = True
+        # create the module
+        self.save_reduce(create_module, (type(obj), pickledModuleName, getattr(obj, "__doc__", None)), module_dict, obj=obj)
+
         if doDel or not reload:
             write(pickle.TUPLE3+pickle.REDUCE)
         return True
-    
+
         
     def _saveMangledModuleName(self, name, module=None):
         """
